@@ -2,13 +2,14 @@
 
 import copy
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, List, Optional, Union, Type, Tuple, Any
+from typing import TYPE_CHECKING, List, Optional, Union, Type, Tuple, Any, cast
+from uuid import uuid4
 
 from sqlfluff.core.errors import SQLParseError
 from sqlfluff.core.string_helpers import curtail_string
 
 from sqlfluff.core.parser.segments import BaseSegment, BracketedSegment, allow_ephemeral
-from sqlfluff.core.parser.helpers import trim_non_code_segments, iter_indices
+from sqlfluff.core.parser.helpers import trim_non_code_segments
 from sqlfluff.core.parser.match_result import MatchResult
 from sqlfluff.core.parser.match_logging import (
     parse_match_logging,
@@ -26,6 +27,28 @@ if TYPE_CHECKING:
     from sqlfluff.core.dialects.base import Dialect  # pragma: no cover
 
 
+def first_trimmed_raw(seg):
+    """Trim whitespace off a whole element raw.
+
+    Used as a helper function in BaseGrammar._look_ahead_match.
+
+    For existing compound segments, we should assume that within
+    that segment, things are internally consistent, that means
+    rather than enumerating all the individual segments of a longer
+    one we just dump out the whole segment, but splitting off the
+    first element separated by whitespace. This is a) faster and
+    also b) prevents some really horrible bugs with bracket matching.
+    See https://github.com/sqlfluff/sqlfluff/issues/433
+
+    This fetches the _whole_ raw of a potentially compound segment
+    to match against, trimming off any whitespace. This is the
+    most efficient way to get at the first element of a potentially
+    longer segment.
+    """
+    s = seg.raw_upper.split(maxsplit=1)
+    return s[0] if s else ""
+
+
 @dataclass
 class BracketInfo:
     """BracketInfo tuple for keeping track of brackets during matching.
@@ -163,6 +186,15 @@ class BaseGrammar(Matchable):
         # If this is the case, the actual segment construction happens in the
         # match_wrapper.
         self.ephemeral_name = ephemeral_name
+        # Generate a cache key
+        self._cache_key = uuid4().hex
+
+    def cache_key(self) -> str:
+        """Get the cache key for this grammar.
+
+        For grammars these are unique per-instance.
+        """
+        return self._cache_key
 
     def is_optional(self):
         """Return whether this segment is optional.
@@ -185,7 +217,7 @@ class BaseGrammar(Matchable):
         )  # pragma: no cover
 
     @cached_method_for_parse_context
-    def simple(self, parse_context: ParseContext, crumbs=None) -> Optional[List[str]]:
+    def simple(self, parse_context: ParseContext, crumbs=None):
         """Does this matcher support a lowercase hash matching route?"""
         return None
 
@@ -218,13 +250,44 @@ class BaseGrammar(Matchable):
         if trim_noncode:
             pre_nc, segments, post_nc = trim_non_code_segments(segments)
 
+        # At parse time we should be able to count on there being a location.
+        assert segments[0].pos_marker
+
+        # Characterise this location.
+        # Initial segment raw, loc, type and length of segment series.
+        loc_key = (
+            segments[0].raw,
+            segments[0].pos_marker.working_loc,
+            segments[0].get_type(),
+            len(segments),
+        )
+
         best_match_length = 0
         # iterate at this position across all the matchers
         for matcher in matchers:
-            # MyPy seems to require a type hint here. Not quite sure why.
-            res_match: MatchResult = matcher.match(
-                segments, parse_context=parse_context
+            # Check parse cache.
+            matcher_key = matcher.cache_key()
+            res_match: Optional[MatchResult] = parse_context.check_parse_cache(
+                loc_key, matcher_key
             )
+            if res_match:
+                parse_match_logging(
+                    cls.__name__,
+                    "_look_ahead_match",
+                    "HIT",
+                    parse_context=parse_context,
+                    cache_hit=matcher.__class__.__name__,
+                    cache_key=matcher_key,
+                )
+            else:
+                # Match fresh if no cache hit
+                res_match = matcher.match(segments, parse_context=parse_context)
+                # Cache it for later to for performance.
+                parse_context.put_parse_cache(loc_key, matcher_key, res_match)
+
+            # By here we know that it's a MatchResult
+            res_match = cast(MatchResult, res_match)
+
             if res_match.is_complete():
                 # Just return it! (WITH THE RIGHT OTHER STUFF)
                 if trim_noncode:
@@ -242,8 +305,11 @@ class BaseGrammar(Matchable):
                     best_match = res_match, matcher
                     best_match_length = res_match.trimmed_matched_length
 
+                    # If we've got a terminator next, it's an opportunity to
+                    # end earlier, and claim an effectively "complete" match.
+                    # NOTE: This means that by specifying terminators, we can
+                    # significantly increase performance.
                     if terminators:
-
                         _, segs, _ = trim_non_code_segments(
                             best_match[0].unmatched_segments
                         )
@@ -317,163 +383,84 @@ class BaseGrammar(Matchable):
         # Here we enable a performance optimisation. Most of the time in this cycle
         # happens in loops looking for simple matchers which we should
         # be able to find a shortcut for.
-        # First: Assess the matchers passed in, if any are
-        # "simple", then we effectively use a hash lookup across the
-        # content of segments to quickly evaluate if the segment is present.
-        # Matchers which aren't "simple" still take a slower route.
-        _matchers = [
-            (matcher, matcher.simple(parse_context=parse_context))
-            for matcher in matchers
-        ]
-        simple_matchers = [matcher for matcher in _matchers if matcher[1]]
-        non_simple_matchers = [matcher[0] for matcher in _matchers if not matcher[1]]
+
+        parse_match_logging(
+            cls.__name__,
+            "_look_ahead_match",
+            "SI",
+            parse_context=parse_context,
+            v_level=4,
+        )
+
         best_simple_match = None
-        if simple_matchers:
-            # If they're all simple we can use a hash match to identify the first one.
-            # Build a buffer of all the upper case raw segments ahead of us.
-            str_buff = []
-            # For existing compound segments, we should assume that within
-            # that segment, things are internally consistent, that means
-            # rather than enumerating all the individual segments of a longer
-            # one we just dump out the whole segment, but splitting off the
-            # first element separated by whitespace. This is a) faster and
-            # also b) prevents some really horrible bugs with bracket matching.
-            # See https://github.com/sqlfluff/sqlfluff/issues/433
-
-            def _trim_elem(seg):
-                s = seg.raw_upper.split(maxsplit=1)
-                return s[0] if s else ""
-
-            str_buff = [_trim_elem(seg) for seg in segments]
-            match_queue = []
-
-            for matcher, simple in simple_matchers:
-                # Simple will be a tuple of options
-                assert simple
-                for simple_option in simple:
-                    # NOTE: We use iter_indices to make sure we capture
-                    # all instances of potential matches if there are many.
-                    # This is important for bracket counting.
-                    for buff_pos in iter_indices(str_buff, simple_option):
-                        match_queue.append((matcher, buff_pos, simple_option))
-
-            # Sort the match queue. First to process AT THE END.
-            # That means we pop from the end.
-            match_queue = sorted(match_queue, key=lambda x: x[1])
-
-            parse_match_logging(
-                cls.__name__,
-                "_look_ahead_match",
-                "SI",
-                parse_context=parse_context,
-                v_level=4,
-                mq=match_queue,
-                sb=str_buff,
-            )
-
-            while match_queue:
-                # We've managed to match. We can shortcut home.
-                # NB: We may still need to deal with whitespace.
-                queued_matcher, queued_buff_pos, queued_option = match_queue.pop()
-                # Here we do the actual transform to the new segment.
-                match = queued_matcher.match(segments[queued_buff_pos:], parse_context)
-                if not match:
-                    # We've had something match in simple matching, but then later
-                    # excluded. Log but then move on to the next item on the list.
-                    parse_match_logging(
-                        cls.__name__,
-                        "_look_ahead_match",
-                        "NM",
-                        parse_context=parse_context,
-                        v_level=4,
-                        _so=queued_option,
+        simple_match = None
+        for idx, seg in enumerate(segments):
+            for matcher in matchers:
+                simple = matcher.simple(parse_context=parse_context)
+                if not simple:  # pragma: no cover
+                    # NOTE: For all bundled dialects, this clause is true, but until
+                    # the RegexMatcher is completely deprecated (and therefore that
+                    # `.simple()` must provide a result), it is still _possible_
+                    # to end up here.
+                    raise NotImplementedError(
+                        "All matchers passed to `._look_ahead_match()` are "
+                        "assumed to have a functioning `.simple()` option. "
+                        "In a future release it will be compulsory for _all_ "
+                        "matchables to implement `.simple()`. Please report "
+                        "this as a bug on GitHub along with your current query "
+                        f"and dialect.\nProblematic matcher: {matcher}"
                     )
-                    continue
-                # Ok we have a match. Because we sorted the list, we'll take it!
-                best_simple_match = (segments[:queued_buff_pos], match, queued_matcher)
+                simple_raws, simple_types = simple
 
-        if not non_simple_matchers:
-            # There are no other matchers, we can just shortcut now.
+                assert simple_raws or simple_types
+                if simple_raws:
+                    trimmed_seg = first_trimmed_raw(seg)
+                    if trimmed_seg in simple_raws:
+                        simple_match = matcher
+                        break
+                if simple_types and not simple_match:
+                    intersection = simple_types.intersection(seg.class_types)
+                    if intersection:
+                        simple_match = matcher
+                        break
 
-            parse_match_logging(
-                cls.__name__,
-                "_look_ahead_match",
-                "SC",
-                parse_context=parse_context,
-                v_level=4,
-                bsm=None
-                if not best_simple_match
-                else (
-                    len(best_simple_match[0]),
-                    len(best_simple_match[1]),
-                    best_simple_match[2],
-                ),
-            )
-
-            if best_simple_match:
-                return best_simple_match
-            else:
-                return ((), MatchResult.from_unmatched(segments), None)
-
-        # Make some buffers
-        seg_buff = segments
-        pre_seg_buff: Tuple[BaseSegment, ...] = ()
-
-        # Loop
-        while True:
-            # Do we have anything left to match on?
-            if seg_buff:
-                # Great, carry on.
-                pass
-            else:
-                # We've got to the end without a match, return empty
-                return ((), MatchResult.from_unmatched(segments), None)
-
-            # We only check the NON-simple ones here for brevity.
-            mat, m = cls._longest_trimmed_match(
-                seg_buff,
-                non_simple_matchers,
-                parse_context=parse_context,
-                trim_noncode=False,
-            )
-
-            if mat and not best_simple_match:
-                return (pre_seg_buff, mat, m)
-            elif mat:
-                # Given we have mat - we should always have these two.
-                assert m
-                assert best_simple_match
-                # It will be earlier than the simple one if we've even checked,
-                # but there's a chance that this might be *longer*, or just FIRST.
-                pre_lengths = (len(pre_seg_buff), len(best_simple_match[0]))
-                mat_lengths = (len(mat), len(best_simple_match[1]))
-                mat_indexes = (matchers.index(m), matchers.index(best_simple_match[2]))
-                if (
-                    (pre_lengths[0] < pre_lengths[1])
-                    or (
-                        pre_lengths[0] == pre_lengths[1]
-                        and mat_lengths[0] > mat_lengths[1]
+            # We've managed to match. We can shortcut home.
+            # NB: We may still need to deal with whitespace.
+            if simple_match:
+                # If we have a _simple_ match, now we should call the
+                # full match method to actually produce the result.
+                match = simple_match.match(segments[idx:], parse_context)
+                if match:
+                    best_simple_match = (
+                        segments[:idx],
+                        match,
+                        simple_match,
                     )
-                    or (
-                        pre_lengths[0] == pre_lengths[1]
-                        and mat_lengths[0] == mat_lengths[1]
-                        and mat_indexes[0] < mat_indexes[1]
-                    )
-                ):
-                    return (pre_seg_buff, mat, m)
+                    break
                 else:
-                    # TODO: Make a test case to cover this.
-                    return best_simple_match  # pragma: no cover
-            else:
-                # If there aren't any matches, then advance the buffer and try again.
-                # Two improvements:
-                # 1) if we get as far as the first simple match, then return that.
-                # 2) be eager in consuming non-code segments if allowed
-                if best_simple_match and len(pre_seg_buff) >= len(best_simple_match[0]):
-                    return best_simple_match
+                    simple_match = None
 
-                pre_seg_buff += (seg_buff[0],)
-                seg_buff = seg_buff[1:]
+        # There are no other matchers, we can just shortcut now. Either with
+        # no match, or the best one we found (if we found one).
+        parse_match_logging(
+            cls.__name__,
+            "_look_ahead_match",
+            "SC",
+            parse_context=parse_context,
+            v_level=4,
+            bsm=None
+            if not best_simple_match
+            else (
+                len(best_simple_match[0]),
+                len(best_simple_match[1]),
+                best_simple_match[2],
+            ),
+        )
+
+        if best_simple_match:
+            return best_simple_match
+        else:
+            return ((), MatchResult.from_unmatched(segments), None)
 
     @classmethod
     def _bracket_sensitive_look_ahead_match(
@@ -816,9 +803,7 @@ class Ref(BaseGrammar):
         super().__init__(*args, **kwargs)
 
     @cached_method_for_parse_context
-    def simple(
-        self, parse_context: ParseContext, crumbs: Optional[Tuple[str]] = None
-    ) -> Optional[List[str]]:
+    def simple(self, parse_context: ParseContext, crumbs: Optional[Tuple[str]] = None):
         """Does this matcher support a uppercase hash matching route?
 
         A ref is simple, if the thing it references is simple.
