@@ -100,6 +100,7 @@ ADD
 ADMIN
 AFTER
 ALLOWED_IP_LIST
+ALLOWED_VALUES
 ALLOW_DUPLICATE
 ALLOW_OVERLAPPING_EXECUTION
 API
@@ -465,6 +466,7 @@ RSA_PUBLIC_KEY_2
 RUNNING
 RUNTIME_VERSION
 S3
+SCALA
 SCALING_POLICY
 SCHEDULE
 SCHEMA
