@@ -3,7 +3,9 @@ import copy
 import json
 import logging
 import os
+import pathlib
 import sys
+import tempfile
 
 from diff_cover.command_runner import execute, run_command_for_code
 from diff_cover.hook import hookimpl as diff_cover_hookimpl
@@ -22,7 +24,7 @@ class SQLFluffDriver(QualityDriver):
 
     def __init__(self):
         super().__init__(
-            "sqlfluff",
+            [sys.executable, "-m", "sqlfluff.cli.commands"],
             [".sql"],
             [
                 s.encode(sys.getfilesystemencoding())
@@ -57,15 +59,23 @@ class SQLFluffViolationReporter(QualityReporter):
         if not self.driver_tool_installed:  # pragma: no cover
             raise OSError(f"{self.driver.name} is not installed")
 
-        output = self.reports if self.reports else self._run_sqlfluff(src_paths)
-        for o in output:
-            # Load and parse SQLFluff JSON output.
-            report = json.loads(o)
-            for file in report:
-                self.violations_dict[file["filepath"]] = [
-                    Violation(v["line_no"], v["description"])
-                    for v in file["violations"]
-                ]
+        if src_paths:
+            output = self.reports if self.reports else self._run_sqlfluff(src_paths)
+            for o in output:
+                # Load and parse SQLFluff JSON output.
+                try:
+                    report = json.loads(o)
+                except json.JSONDecodeError as e:  # pragma: no cover
+                    print(f"Error parsing JSON output ({e}): {repr(o)}")
+                    raise
+                else:
+                    for file in report:
+                        self.violations_dict[file["filepath"]] = [
+                            Violation(v["line_no"], v["description"])
+                            for v in file["violations"]
+                        ]
+        else:
+            logger.warning("Not running SQLFluff: No files to check")
         return self.violations_dict
 
     def _run_sqlfluff(self, src_paths):
@@ -78,16 +88,30 @@ class SQLFluffViolationReporter(QualityReporter):
             if src_path.endswith(".sql") and os.path.exists(src_path):
                 command.append(src_path.encode(sys.getfilesystemencoding()))
 
-        # Run SQLFluff.
-        printable_command = " ".join(
-            [
-                c.decode(sys.getfilesystemencoding()) if isinstance(c, bytes) else c
-                for c in command
-            ]
-        )
-        logger.warning(f"{printable_command}")
-        output = execute(command, self.driver.exit_codes)
-        return [output[1]] if self.driver.output_stderr else [output[0]]
+        with tempfile.NamedTemporaryFile(
+            prefix="sqlfluff-", suffix=".json", delete=False
+        ) as f:
+            f.close()
+            try:
+                # Write output to a temporary file. This avoids issues where
+                # extraneous SQLFluff or dbt output results in the JSON output
+                # being invalid.
+                command += ["--write-output", f.name]
+
+                # Run SQLFluff.
+                printable_command = " ".join(
+                    [
+                        c.decode(sys.getfilesystemencoding())
+                        if isinstance(c, bytes)
+                        else c
+                        for c in command
+                    ]
+                )
+                logger.warning(f"{printable_command}")
+                execute(command, self.driver.exit_codes)
+                return [pathlib.Path(f.name).read_text()]
+            finally:
+                os.remove(f.name)
 
     def measured_lines(self, src_path: str) -> None:  # pragma: no cover
         """Return list of the lines in src_path that were measured."""
